using System.ComponentModel;
using Dapfor.Net.Data;

namespace Dapfor.Net.Grid.BasketViewer.BusinessLevel
{
    internal class Basket
    {
        private readonly ThreadSafeBindingList<InstrumentClass> _instrumentClasses = new ThreadSafeBindingList<InstrumentClass>();
        private decimal? _basketValue;

        public ThreadSafeBindingList<InstrumentClass> InstrumentClasses
        {
            get { return _instrumentClasses; }
        }

        public decimal BasketValue
        {
            get
            {
                //Cache purposes: compute industry's market value
                if (_basketValue == null)
                {
                    _basketValue = 0;
                    foreach (InstrumentClass share in _instrumentClasses)
                    {
                        _basketValue += share.MarketValue;
                    }
                }
                return _basketValue.Value;
            }
        }

        public void AddIndustryType(InstrumentClass instrumentClass)
        {
            if (instrumentClass != null)
            {
                _instrumentClasses.Add(instrumentClass);
                instrumentClass.PropertyChanged += OnIndustryTypePropertyChanged;
                _basketValue = null;
            }
        }

        private void OnIndustryTypePropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "MarketValue":
                    _basketValue = null;
                    //TODO: Fire notification here about affected fields
                    break;
            }
        }
    }
}