using System.Collections.Generic;
using System.ComponentModel;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Grid.BasketViewer.BusinessLevel
{
    internal class Share : INotifyPropertyChanged
    {
        public const int MaxHistoryLength = 40;

        private readonly Industry _industry;
        private readonly string _isin;
        private readonly string _name;
        private readonly long _quantity;
        private decimal _lastPrice;
        private decimal _price;
        private readonly List<decimal> _history = new List<decimal>();

        public Share(string name, string isin, Industry industry, decimal price, long quantity)
        {
            _industry = industry;
            _name = name;
            _isin = isin;
            _lastPrice = price;
            _price = price;
            _quantity = quantity;
        }

        #region Public properties

        public string Name
        {
            get { return _name; }
        }

        public string Isin
        {
            get { return _isin; }
        }

        [Format("### ### ###.00")]
        public decimal LastPrice
        {
            get { return _lastPrice; }
        }

        [Format("### ### ###.00")]
        public decimal Price
        {
            get { return _price; }
            set
            {
                if (!Equals(_price, value))
                {
                    _lastPrice = _price;
                    _price = value;
                    _history.Insert(0, value);
                    if (_history.Count > MaxHistoryLength)
                    {
                        _history.RemoveAt(MaxHistoryLength);
                    }
                    FirePropertyChanged("Price");
                    FirePropertyChanged("LastPrice");
                    FirePropertyChanged("MarketValue");
                    FirePropertyChanged("MarketValueEur");
                    FirePropertyChanged("History");
                }
            }
        }

        [Format("### ### ###")]
        public long Quantity
        {
            get { return _quantity; }
        }

        public string ClassName
        {
            get { return _industry.InstrumentClass.InstrumentClassName; }
        }

        public decimal MarketPart
        {
            get
            {
                if (_industry.MarketPart != 0)
                {
                    return _industry.MarketPart*MarketValue/_industry.MarketValue;
                }
                return 0;
            }
        }

        [Format("#.00", null, " %")]
        public decimal MarketPartPercent
        {
            get { return 100*MarketPart; }
        }

        //Use 2 decimals for shares 
        [Format("### ### ###.00", null, " ")]
        public decimal MarketValueEur
        {
            get { return MarketValue; }
        }

        [Format("### ### ###.00")]
        public decimal MarketValue
        {
            get { return _quantity*_price; }
        }

        public IList<decimal> History
        {
            get { return _history; }
        }


        #endregion

        #region INotifyPropertyChanged implementation

        public event PropertyChangedEventHandler PropertyChanged;

        private void FirePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion
    }
}